@@ -5,6 +5,7 @@ use crate::configuration::*;
 use dprint_core::formatting::*;
 use dprint_core::formatting::{condition_resolvers, conditions::*, parser_helpers::*};
 use std::borrow::Cow;
+use utils::get_leading_non_space_tab_byte_pos;
 
 pub fn parse_node(node: &Node, context: &mut Context) -> PrintItems {
   // println!("Kind: {:?}", node.kind());
@@ -186,7 +187,12 @@ fn parse_nodes(nodes: &Vec<Node>, context: &mut Context) -> PrintItems {
           if utils::has_leading_blankline(node.range().start, context.file_text) {
             items.push_signal(Signal::NewLine);
           }
-          items.extend(parser_helpers::parse_raw_string(node.text(context).trim_end()));
+
+          // include the leading indent
+          let range = node.range();
+          let text_start = get_leading_non_space_tab_byte_pos(context.file_text, range.start);
+          items.extend(parser_helpers::parse_raw_string(context.file_text[text_start..range.end].trim_end()));
+
           last_node = Some(node);
         }
       } else if context.ignore_start_regex.is_match(&html.text) {
@@ -202,19 +208,19 @@ fn parse_nodes(nodes: &Vec<Node>, context: &mut Context) -> PrintItems {
             }
           }
 
-          if let Some(taken_range) = range {
-            range = Some(Range {
-              start: taken_range.start,
-              end: node.range().end,
-            });
-          } else {
-            range = Some(node.range().to_owned());
-          }
+          let node_range = node.range();
+          range = Some(Range {
+            start: range.map(|r| r.start).unwrap_or(node_range.start),
+            end: node_range.end,
+          });
         }
 
         if let Some(range) = range {
           items.extend(get_conditional_blank_line(&range, context));
-          items.extend(parser_helpers::parse_raw_string(&context.file_text[range.start..range.end].trim()));
+          // get the leading indent
+          let text_start = get_leading_non_space_tab_byte_pos(context.file_text, range.start);
+          items.extend(parser_helpers::parse_raw_string(&context.file_text[text_start..range.end].trim_end()));
+
           if let Some(end_comment) = end_comment {
             items.extend(get_conditional_blank_line(end_comment.range(), context));
             items.extend(parse_html(end_comment, context));
@@ -239,7 +245,7 @@ fn parse_nodes(nodes: &Vec<Node>, context: &mut Context) -> PrintItems {
 fn parse_heading(heading: &Heading, context: &mut Context) -> PrintItems {
   let mut items = PrintItems::new();
 
-  items.push_str(&format!("{} ", "#".repeat(heading.level as usize)));
+  items.push_string(format!("{} ", "#".repeat(heading.level as usize)));
   items.extend(with_no_new_lines(parse_nodes(&heading.children, context)));
 
   items
@@ -288,7 +294,7 @@ fn parse_code_block(code_block: &CodeBlock, context: &mut Context) -> PrintItems
 
   // header
   if code_block.is_fenced {
-    items.push_str(&backtick_text);
+    items.push_string(backtick_text.clone());
     if let Some(tag) = &code_block.tag {
       items.push_str(tag);
     }
@@ -305,7 +311,7 @@ fn parse_code_block(code_block: &CodeBlock, context: &mut Context) -> PrintItems
     if !code_text.is_empty() {
       items.push_signal(Signal::NewLine);
     }
-    items.push_str(&backtick_text);
+    items.push_string(backtick_text);
   }
 
   return with_indent_times(items, relative_indent_level);
@@ -424,7 +430,7 @@ fn parse_text(text: &Text, context: &mut Context) -> PrintItems {
           }
         }
 
-        self.items.push_str(&current_word);
+        self.items.push_string(current_word);
         self.was_last_newline = false;
       }
     }
@@ -445,9 +451,9 @@ fn parse_text_decoration(text: &TextDecoration, context: &mut Context) -> PrintI
     TextDecorationKind::Strikethrough => "~~",
   };
 
-  items.push_str(&decoration_text);
+  items.push_str(decoration_text);
   items.extend(parse_nodes(&text.children, context));
-  items.push_str(&decoration_text);
+  items.push_str(decoration_text);
 
   items
 }
@@ -460,84 +466,93 @@ fn parse_html(html: &Html, _: &mut Context) -> PrintItems {
 
 fn parse_footnote_reference(footnote_reference: &FootnoteReference, _: &mut Context) -> PrintItems {
   let mut items = PrintItems::new();
-  items.push_str(&format!("[^{}]", footnote_reference.name.trim()));
+  items.push_string(format!("[^{}]", footnote_reference.name.trim()));
   parser_helpers::with_no_new_lines(items)
 }
 
 fn parse_footnote_definition(footnote_definition: &FootnoteDefinition, context: &mut Context) -> PrintItems {
   let mut items = PrintItems::new();
-  items.push_str(&format!("[^{}]: ", footnote_definition.name.trim()));
+  items.push_string(format!("[^{}]: ", footnote_definition.name.trim()));
   items.extend(parse_nodes(&footnote_definition.children, context));
   items
 }
 
 fn parse_inline_link(link: &InlineLink, context: &mut Context) -> PrintItems {
-  let mut items = PrintItems::new();
-  let parsed_children = parse_nodes(&link.children, context);
-  items.push_str("[");
+  context.with_no_text_wrap(|context| {
+    let mut items = PrintItems::new();
+    let parsed_children = parse_nodes(&link.children, context);
+    items.push_str("[");
 
-  // force the text to be on a single line in some scenarios
-  let (parsed_children, parsed_children_clone) = clone_items(parsed_children);
-  let single_line_text = get_items_text(parser_helpers::with_no_new_lines(parsed_children_clone));
-  if single_line_text.len() < (context.configuration.line_width / 2) as usize {
-    items.push_str(&single_line_text);
-  } else {
-    items.extend(parsed_children);
-  }
+    // force the text to be on a single line in some scenarios
+    let (parsed_children, parsed_children_clone) = clone_items(parsed_children);
+    let single_line_text = get_items_text(parser_helpers::with_no_new_lines(parsed_children_clone));
+    if single_line_text.len() < (context.configuration.line_width / 2) as usize {
+      items.push_string(single_line_text);
+    } else {
+      items.extend(parsed_children);
+    }
 
-  items.push_str("]");
-  items.push_str("(");
-  items.push_str(&link.url.trim());
-  if let Some(title) = &link.title {
-    items.push_str(&format!(" \"{}\"", title.trim()));
-  }
-  items.push_str(")");
+    items.push_str("]");
+    items.push_str("(");
+    items.push_str(link.url.trim());
+    if let Some(title) = &link.title {
+      items.push_string(format!(" \"{}\"", title.trim()));
+    }
+    items.push_str(")");
 
-  parser_helpers::new_line_group(items)
+    parser_helpers::new_line_group(items)
+  })
 }
 
 fn parse_reference_link(link: &ReferenceLink, context: &mut Context) -> PrintItems {
-  let mut items = PrintItems::new();
-  items.push_str("[");
-  items.extend(parse_nodes(&link.children, context));
-  items.push_str("]");
-  items.push_str(&format!("[{}]", link.reference.trim()));
-  parser_helpers::new_line_group(items)
+  context.with_no_text_wrap(|context| {
+    let mut items = PrintItems::new();
+    items.push_str("[");
+    items.extend(parse_nodes(&link.children, context));
+    items.push_str("]");
+    items.push_string(format!("[{}]", link.reference.trim()));
+    parser_helpers::new_line_group(items)
+  })
 }
 
 fn parse_shortcut_link(link: &ShortcutLink, context: &mut Context) -> PrintItems {
-  let mut items = PrintItems::new();
-  items.push_str("[");
-  items.extend(parse_nodes(&link.children, context));
-  items.push_str("]");
-  parser_helpers::new_line_group(items)
+  context.with_no_text_wrap(|context| {
+    let mut items = PrintItems::new();
+    items.push_str("[");
+    items.extend(parse_nodes(&link.children, context));
+    items.push_str("]");
+    parser_helpers::new_line_group(items)
+  })
 }
 
 fn parse_auto_link(link: &AutoLink, context: &mut Context) -> PrintItems {
-  let mut items = PrintItems::new();
-  items.push_str("<");
-  items.extend(parse_nodes(&link.children, context));
-  items.push_str(">");
-  parser_helpers::new_line_group(items)
+  // auto-links can't contain spaces, but do this anyway just in case
+  context.with_no_text_wrap(|context| {
+    let mut items = PrintItems::new();
+    items.push_str("<");
+    items.extend(parse_nodes(&link.children, context));
+    items.push_str(">");
+    parser_helpers::new_line_group(items)
+  })
 }
 
 fn parse_link_reference(link_ref: &LinkReference, _: &mut Context) -> PrintItems {
   let mut items = PrintItems::new();
-  items.push_str(&format!("[{}]: ", link_ref.name.trim()));
-  items.push_str(&link_ref.link.trim());
+  items.push_string(format!("[{}]: ", link_ref.name.trim()));
+  items.push_str(link_ref.link.trim());
   if let Some(title) = &link_ref.title {
-    items.push_str(&format!(" \"{}\"", title.trim()));
+    items.push_string(format!(" \"{}\"", title.trim()));
   }
   parser_helpers::new_line_group(items)
 }
 
 fn parse_inline_image(image: &InlineImage, _: &mut Context) -> PrintItems {
   let mut items = PrintItems::new();
-  items.push_str(&format!("![{}]", image.text.trim()));
+  items.push_string(format!("![{}]", image.text.trim()));
   items.push_str("(");
-  items.push_str(&image.url.trim());
+  items.push_str(image.url.trim());
   if let Some(title) = &image.title {
-    items.push_str(&format!(" \"{}\"", title.trim()));
+    items.push_string(format!(" \"{}\"", title.trim()));
   }
   items.push_str(")");
   parser_helpers::new_line_group(items)
@@ -545,52 +560,52 @@ fn parse_inline_image(image: &InlineImage, _: &mut Context) -> PrintItems {
 
 fn parse_reference_image(image: &ReferenceImage, _: &mut Context) -> PrintItems {
   let mut items = PrintItems::new();
-  items.push_str(&format!("![{}]", image.text.trim()));
-  items.push_str(&format!("[{}]", image.reference.trim()));
+  items.push_string(format!("![{}]", image.text.trim()));
+  items.push_string(format!("[{}]", image.reference.trim()));
   parser_helpers::new_line_group(items)
 }
 
 fn parse_list(list: &List, is_alternate: bool, context: &mut Context) -> PrintItems {
-  let mut items = PrintItems::new();
-  context.is_in_list_count += 1;
-  let measured_raw_indent_level = context.get_indent_level_at_pos(list.range.start);
-  let raw_list_start_indent_increment = utils::safe_subtract_to_zero(measured_raw_indent_level, context.raw_indent_level);
-  context.raw_indent_level += raw_list_start_indent_increment;
+  context.mark_in_list(|context| {
+    let mut items = PrintItems::new();
+    let measured_raw_indent_level = context.get_indent_level_at_pos(list.range.start);
+    let raw_list_start_indent_increment = utils::safe_subtract_to_zero(measured_raw_indent_level, context.raw_indent_level);
+    context.raw_indent_level += raw_list_start_indent_increment;
 
-  // parse items
-  for (index, child) in list.children.iter().enumerate() {
-    if index > 0 {
-      items.push_signal(Signal::NewLine);
-      if utils::has_leading_blankline(child.range().start, context.file_text) {
+    // parse items
+    for (index, child) in list.children.iter().enumerate() {
+      if index > 0 {
         items.push_signal(Signal::NewLine);
+        if utils::has_leading_blankline(child.range().start, context.file_text) {
+          items.push_signal(Signal::NewLine);
+        }
       }
+      let prefix_text = if let Some(start_index) = list.start_index {
+        let end_char = if is_alternate { ")" } else { "." };
+        let display_index = if is_all_ones_list(list, context) { 1 } else { start_index + index as u64 };
+        format!("{}{}", display_index, end_char)
+      } else {
+        String::from(if is_alternate { "*" } else { "-" })
+      };
+      let indent_increment = (prefix_text.chars().count() + 1) as u32;
+      context.indent_level += indent_increment;
+      context.raw_indent_level += indent_increment;
+      items.push_string(prefix_text);
+      let after_child = Info::new("afterChild");
+      items.push_condition(if_true(
+        "spaceIfHasChild",
+        move |context| Some(!condition_resolvers::is_at_same_position(context, &after_child)?),
+        Signal::SpaceIfNotTrailing.into(),
+      ));
+      items.extend(with_indent_times(parse_node(child, context), indent_increment));
+      items.push_info(after_child);
+      context.indent_level -= indent_increment;
+      context.raw_indent_level -= indent_increment;
     }
-    let prefix_text = if let Some(start_index) = list.start_index {
-      let end_char = if is_alternate { ")" } else { "." };
-      let display_index = if is_all_ones_list(list, context) { 1 } else { start_index + index as u64 };
-      format!("{}{}", display_index, end_char)
-    } else {
-      String::from(if is_alternate { "*" } else { "-" })
-    };
-    let indent_increment = (prefix_text.chars().count() + 1) as u32;
-    context.indent_level += indent_increment;
-    context.raw_indent_level += indent_increment;
-    items.push_str(&prefix_text);
-    let after_child = Info::new("afterChild");
-    items.push_condition(if_true(
-      "spaceIfHasChild",
-      move |context| Some(!condition_resolvers::is_at_same_position(context, &after_child)?),
-      Signal::SpaceIfNotTrailing.into(),
-    ));
-    items.extend(with_indent_times(parse_node(child, context), indent_increment));
-    items.push_info(after_child);
-    context.indent_level -= indent_increment;
-    context.raw_indent_level -= indent_increment;
-  }
 
-  context.raw_indent_level -= raw_list_start_indent_increment;
-  context.is_in_list_count -= 1;
-  items
+    context.raw_indent_level -= raw_list_start_indent_increment;
+    items
+  })
 }
 
 fn parse_item(item: &Item, context: &mut Context) -> PrintItems {
@@ -677,7 +692,7 @@ fn parse_table(table: &Table, context: &mut Context) -> PrintItems {
       if column_alignment_props.has_left_colon {
         items.push_str(":");
       }
-      items.push_str(&"-".repeat(dashes_count));
+      items.push_string("-".repeat(dashes_count));
       if column_alignment_props.has_right_colon {
         items.push_str(":");
       }
@@ -705,11 +720,11 @@ fn parse_table(table: &Table, context: &mut Context) -> PrintItems {
           ColumnAlignment::None | ColumnAlignment::Left => {}
           ColumnAlignment::Center => {
             if difference > 1 {
-              items.push_str(&" ".repeat((difference as f32 / 2_f32).floor() as usize))
+              items.push_string(" ".repeat((difference as f32 / 2_f32).floor() as usize))
             }
           }
           ColumnAlignment::Right => {
-            items.push_str(&" ".repeat(difference));
+            items.push_string(" ".repeat(difference));
           }
         }
       }
@@ -719,9 +734,9 @@ fn parse_table(table: &Table, context: &mut Context) -> PrintItems {
       if difference > 0 {
         match column_alignment {
           ColumnAlignment::None | ColumnAlignment::Left => {
-            items.push_str(&" ".repeat(difference));
+            items.push_string(" ".repeat(difference));
           }
-          ColumnAlignment::Center => items.push_str(&" ".repeat((difference as f32 / 2_f32).ceil() as usize)),
+          ColumnAlignment::Center => items.push_string(" ".repeat((difference as f32 / 2_f32).ceil() as usize)),
           ColumnAlignment::Right => {}
         }
       }
@@ -832,6 +847,9 @@ fn get_items_text(items: PrintItems) -> String {
 }
 
 fn get_space_or_newline_based_on_config(context: &Context) -> PrintItems {
+  if context.is_text_wrap_disabled() {
+    return " ".into();
+  }
   match context.configuration.text_wrap {
     TextWrap::Always => Signal::SpaceOrNewLine.into(),
     TextWrap::Never | TextWrap::Maintain => " ".into(),
